<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><title>Healthcheck Report</title><meta name="viewport" content="width=device-width,initial-scale=1"><link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/github-markdown-css/5.8.1/github-markdown.min.css"><script src="https://cdn.jsdelivr.net/npm/chart.js"></script><script type="text/javascript">function getWindowHeight(){return window.innerHeight||document.documentElement.clientHeight||document.body.clientHeight}function getWindowWidth(){return window.innerWidth||document.documentElement.clientWidth||document.body.clientWidth}function highLightOutline(){for(var e=0;e<titleElms.length;e++){var t=titleElms[e].getBoundingClientRect(),n=0==e?0:t.top,i=e==titleElms.length-1?document.body.scrollHeight:titleElms[e+1].getBoundingClientRect().top;n<1&&i>1?linkElms[e].classList.add("in-view"):linkElms[e].classList.remove("in-view")}}function createList(e){for(var t=document.createElement("ul"),n=0;n<e.length;n++){var i=document.createElement("li"),l=document.createElement("a");linkElms.push(l),l.href="#"+e[n].id,l.textContent=e[n].textContent,i.appendChild(l),t.appendChild(i)}return t}function formatSize(e,t=2){const n=["B","KB","MB","GB","TB"];for(let i of n){if(e<1024)return e.toFixed(t)+" "+i;e/=1024}return e.toFixed(t)+" PB"}VIEWPORT_WIDTH=1200,linkElms=[],titleElms=[],charts=[],window.onload=function(){for(var e=document.getElementById("outline"),t=document.getElementsByTagName("h2"),n=document.createElement("ul"),i=document.getElementsByClassName("markdown-body")[0],l=0;l<t.length;l++){titleElms.push(t[l]);var o=document.createElement("li"),a=document.createElement("a");linkElms.push(a),a.href="#"+t[l].id,a.textContent=t[l].textContent;for(var s=[],d=t[l].nextElementSibling;d&&"H2"!==d.tagName;d=d.nextElementSibling)"H3"===d.tagName&&(s.push(d),titleElms.push(d));n.appendChild(o),o.appendChild(a),o.appendChild(createList(s))}e.appendChild(n);var c=e.offsetWidth;i.style.margin="0 auto 0 "+c+"px",expand=document.getElementById("expand-outline"),collapse=document.getElementById("collapse-outline"),b=e.getElementsByTagName("b")[0],state=1,states=[["none","block"],["none","inline"],["inline","none"],["none","inline"],["0 auto","0 auto 0 "+c+"px"]],window.onresize=function(){blank=(getWindowWidth()-VIEWPORT_WIDTH)/2,blank>c?states=[["none","block"],["none","inline"],["inline","none"],["none","inline"],["0 auto","0 auto 0 auto"]]:states=[["none","block"],["none","inline"],["inline","none"],["none","inline"],["0 auto","0 auto 0 "+c+"px"]],i.style.margin=states[4][state];for(var e=0;e<charts.length;e++)charts[e].resize()},click=function(){return state=0==state?1:0,n.style.display=states[0][state],b.style.display=states[1][state],expand.style.display=states[2][state],collapse.style.display=states[3][state],i.style.margin=states[4][state],!1},collapse.onclick=click,expand.onclick=click,highLightOutline(),window.onscroll=highLightOutline}</script><style>.markdown-body{box-sizing:border-box;min-width:200px;max-width:1200px;margin:0 auto 0 350px;padding:45px}@media(max-width:90%){.markdown-body{padding:15px}}#outline{position:fixed;top:10px;left:10px;padding:40px 0}.pie{display:inline-block;width:30%}.bar{display:inline-block;width:100%}a.in-view{background-color:#eee;font-weight:700}</style></head><body class="markdown-body"><div id="outline"><a href="#" id="collapse-outline">&#x25C0;</a> <b>Report Outline</b> <a href="#" style="display:none" id="expand-outline">&#x25B6;</a></div>{{ content }}</body></html>