<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><title>Healthcheck Report</title><meta name="viewport" content="width=device-width,initial-scale=1"><link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/github-markdown-css/5.8.1/github-markdown.min.css"><style>.markdown-body{box-sizing:border-box;min-width:200px;max-width:980px;margin:0 auto;padding:45px}@media(max-width:90%){.markdown-body{padding:15px}}</style></head><body class="markdown-body">{{ content }}</body></html>